/**************************************************************************//**
*   @file   ad5235.c
*   @brief  Implementation of AD5235 Driver for Microblaze processor.
*   @author ACozma (andrei.cozma@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 468
******************************************************************************/
#define AD5235_GLOBALS

/*****************************************************************************/
/* Include Files                                                             */
/*****************************************************************************/
#include "ad5235.h"
#include "../drivers/spi.h"
#include "../drivers/gpio.h"

/*****************************************************************************/
/*  Private variables                                                        */
/*****************************************************************************/
unsigned char   txBuffer[4]     = {0,};     // Tx SPI Buffer.
unsigned char   rxBuffer[4]     = {0,};     // Rx SPI Buffer.

/*****************************************************************************/
/* Aliases for uC/Probe data                                                 */
/*****************************************************************************/
char*           restoreRdac1    = &AD5235_Data[0].chData;   //used as flag for Restore button, RDAC1
char*           storeRdac1      = &AD5235_Data[1].chData;   //used as flag for Store button, RDAC1
char*           restoreRdac2    = &AD5235_Data[2].chData;   //used as flag for Restore button, RDAC2
char*           storeRdac2      = &AD5235_Data[3].chData;   //used as flag for Store button, RDAC2
char*           writeEem        = &AD5235_Data[4].chData;   //used as flag for Write button, memory section
char*           readEem         = &AD5235_Data[5].chData;   //used as flag for Read button, memory section
char*           resetRdacs      = &AD5235_Data[6].chData;   //used as flag for generic RESET button, RDAC section
char*           writeRdac1      = &AD5235_Data[7].chData;   //used as flag for Write button, RDAC1
char*           writeRdac2      = &AD5235_Data[8].chData;   //used as flag for Write button, RDAC2
char*           incRdac1        = &AD5235_Data[9].chData;   //used as flag for +1 step button, RDAC1
char*           decRdac1        = &AD5235_Data[10].chData;  //used as flag for -1 step button, RDAC1
char*           incRdac2        = &AD5235_Data[11].chData;  //used as flag for +1 step button, RDAC2
char*           decRdac2        = &AD5235_Data[12].chData;  //used as flag for -1 step button, RDAC2
char*           inc6dBRdac1     = &AD5235_Data[13].chData;  //used as flag for +6dB step button, RDAC1
char*           dec6dBRdac1     = &AD5235_Data[14].chData;  //used as flag for -6dB step button, RDAC1
char*           inc6dBRdac2     = &AD5235_Data[15].chData;  //used as flag for +6dB step button, RDAC2
char*           dec6dBRdac2     = &AD5235_Data[16].chData;  //used as flag for -6dB step button, RDAC2
char*           incRdacs        = &AD5235_Data[17].chData;  //used as flag for +1 step button, both RDACs
char*           decRdacs        = &AD5235_Data[18].chData;  //used as flag for -1 step button, both RDACs
char*           inc6dBRdacs     = &AD5235_Data[19].chData;  //used as flag for +6dB step button, both RDACs
char*           dec6dBRdacs     = &AD5235_Data[20].chData;  //used as flag for -6dB step button, both RDACs
char*           toleranceFlag   = &AD5235_Data[21].chData;  //used as flag for tolerance read and computation
char*           cmdFlag         = &AD5235_Data[22].chData;  //used as flag for SendCommand button
char*           eemWp           = &AD5235_Data[23].chData;  //used as flag for Hardware Write Protect button
char*           pr              = &AD5235_Data[24].chData;  //used as flag for Hardware \PR button
unsigned char*  eemWrAddr       = &AD5235_Data[25].uchData; //used to store the Write Command Address for Memory Write
unsigned short* eemWrData       = &AD5235_Data[26].usiData; //used for Write functions both to Memory and RDAC1/RDAC2
unsigned char*  command         = &AD5235_Data[27].uchData; //used for storing the cmd value, sent with a generic cmd
unsigned char*  cmdAddr         = &AD5235_Data[28].uchData; //used for storing the addr value sent with a generic cmd
unsigned short* cmdData         = &AD5235_Data[29].usiData; //used for storing the data value sent with a generic cmd
unsigned short* cmdResponseS    = &AD5235_Data[30].usiData; //used for storing the data section  from a response
unsigned char*  cmdResponseC    = &AD5235_Data[31].uchData; //used for storing the command section from a response
unsigned char*  cmdAddrResponse = &AD5235_Data[32].uchData; //used for storing the address section from a response
float*          tolerance       = &AD5235_Data[33].fData;   //used to store the computed tolerance

/**************************************************************************//**
 * @brief Initialize SPI and Initial Values for AD5235 Board.
 *        At initialization a tolerance read and computation will be performed
 *
 * @param None.
 *
 * @return success if the communication with the AD5235 works correct;
 *         fail otherwise.
******************************************************************************/
int AD5235_Init(void)
{
    unsigned long  count;

    GPIO_SET_OUTPUT_PINS(XPAR_AXI_GPIO_0_BASEADDR, ~(GPIO_RESET | GPIO_WP | GPIO_MUX_A0 | GPIO_MUX_A1 | GPIO_MUX_A2));
    GPIO_SET_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_RESET);                // set RESET to HIGH (active LOW)
    GPIO_SET_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_WP);                   // set WP to HIGH (active LOW)

    for (count = 0; count < 10000000; count++)
    {
        asm("nop");
    }

    GPIO_SET_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_MUX_A2 | GPIO_MUX_A1 ); // select the AD5235 SPI line (SYNC-7) as output of SYNC lines MUX
    GPIO_CLEAR_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_MUX_A0 ); // select the AD5235 SPI line (SYNC-7) as output of SYNC lines MUX

    for (count = 0; count < 10000000; count++)
    {
        asm("nop");
    }
    SPI_Init(AD5235_SPI, 0, 1, 1);

    *toleranceFlag = 1;
    //check if communication with the AD5235 is active
    AD5235_SendCommand( READ_RDAC_CMD, 0, NULL, cmdResponseS ); //send Read RDAC1 Wiper settings
    *cmdResponseS = 0;                                          //clean the command response
    if((rxBuffer[0]) != 160)
    {
        return(FALSE);
    }

    return(TRUE);
}

/**************************************************************************//**
 * @brief Send a command to the device. It will initially transmit 3 bytes and then read 3 bytes (if it has output data as parameter)
 *
 * @param command - will be used to select the command that will be sent (valid values 0-15)
 * @param address - will be used to set the address section of a command (valid values 0-15)
 * @param inData - will be used to set the data section of a command (valid values 0-65535)
 * @param outData - will be used to return output data/data read from the SDO)
 *
 * @return None
******************************************************************************/
void AD5235_SendCommand(unsigned char command, unsigned char address,
                        unsigned short* inData, unsigned short* outData)
{
    long count;
    long delay;
    txBuffer[0] = (unsigned char) ((command << 4) | (address));
    txBuffer[1] = inData ? (unsigned char) ((*inData >> 8) & 0x0FF) : 0;
    txBuffer[2] = inData ? (unsigned char) ((*inData) & 0x0FF) : 0;
    for (count = 0; count < 3000; count++)
    {
        asm("nop");
    }
    //send command to the device
    SPI_TransferData(AD5235_SPI, 3, (char * ) txBuffer, 0, NULL, 1);
    //if the output buffer is not NULL read data from the device
    if (outData)
    {
        if (command == STORE_RDAC_CMD || command == WRITE_EEM_CMD)
        {
            delay = 5000000;
        }
        else
        {
            delay = 30000;
        }
        for (count = 0; count < 100000; count++)
        {
            asm("nop");
        }
        if (!SPI_TransferData(AD5235_SPI, 0, NULL, 3, (char *) rxBuffer, 1))
        {
            SPI_Init(AD5235_SPI, 0, 1, 1);
        }
        *outData = ((unsigned short) rxBuffer[1] << 8)
                 | ((unsigned short) rxBuffer[2]);
    }
}

/**************************************************************************//**
 * @brief Update the device's monitored and controlled parameters
 *
 * @param None
 *
 * @return Always returns success
******************************************************************************/
int AD5235_UpdateParameters(void)
{
    int i;
    unsigned char tempCmd    = 0;   //used for intermediary computation;
                                    //needed in order to avoid display of intermediary values
    unsigned char tempAddr   = 0;   //used for intermediary computation
    unsigned short tempCmdData = 0; //used for intermediary computation
    for (i = 0; i < 4; i++)
    {
        tempCmd += rgCmd[i] << i;
    }
    for (i = 0; i < 4; i++)
    {
        tempAddr += rgCmdAddr[i] << i;
    }
    for (i = 0; i < 16; i++)
    {
        tempCmdData += rgCmdData[i] << i;
    }
    *command=tempCmd;
    *cmdAddr=tempAddr;
    *cmdData=tempCmdData;
    AD5235_SendCommand((unsigned char) READ_RDAC_CMD, 1, NULL, &rgRdacVal[1]);
    AD5235_SendCommand((unsigned char) READ_RDAC_CMD, 0, NULL, &rgRdacVal[0]);
    //check RESTORE RDAC1 button
    if (*restoreRdac1)
    {
        AD5235_SendCommand((unsigned char) RESTORE_RDAC_CMD, 0, NULL, NULL);
        *restoreRdac1 = 0;
    }
    //check STORE RDAC1 button
    if (*storeRdac1)
    {
        AD5235_SendCommand((unsigned char) STORE_RDAC_CMD, 0, NULL, NULL);
        *storeRdac1 = 0;
    }
    //check RESTORE RDAC2 button
    if (*restoreRdac2)
    {
        AD5235_SendCommand((unsigned char) RESTORE_RDAC_CMD, 1, NULL, NULL);
        *restoreRdac2 = 0;
    }
    //check STORE RDAC2 button
    if (*storeRdac2)
    {
        AD5235_SendCommand((unsigned char) STORE_RDAC_CMD, 1, NULL, NULL);
        *storeRdac2 = 0;
    }
    //check Write Memory Button
    if (*writeEem)
    {
        AD5235_SendCommand((unsigned char) WRITE_EEM_CMD, *eemWrAddr, eemWrData, NULL);
        *writeEem = 0;
    }
    //check Read Memory button. It will read all the memory locations on the AD5235
    if (*readEem)
    {
        unsigned char i;
        for (i = 0; i < EEM_COUNT; i++)
        {
            AD5235_SendCommand((unsigned char) READ_EEM_CMD, i, NULL, &rgEemVal[i]);
        }
        *readEem = 0;
    }
    //check Reset RDACs button
    if (*resetRdacs)
    {
        AD5235_SendCommand((unsigned char) RESET_RDACS_CMD, 0, NULL, NULL);
        *resetRdacs = 0;
    }
    //check Write RDAC1 button
    if (*writeRdac1)
    {
        AD5235_SendCommand((unsigned char) WRITE_RDAC_CMD, 0, eemWrData, NULL);
        *writeRdac1 = 0;
    }
    //check Write RDAC2 button
    if (*writeRdac2)
    {
        AD5235_SendCommand((unsigned char) WRITE_RDAC_CMD, 1, eemWrData, NULL);
        *writeRdac2 = 0;
    }
    //check +6dB RDAC1 button
    if ((unsigned char) *inc6dBRdac1)
    {
        AD5235_SendCommand((unsigned char) INCREMENT_6dB_CMD, 0, NULL, NULL);
        *inc6dBRdac1 = 0;
    }
    //check -6dB RDAC1 button
    if ((unsigned char) *dec6dBRdac1)
    {
        AD5235_SendCommand((unsigned char) DECREMENT_6dB_CMD, 0, NULL, NULL);
        *dec6dBRdac1 = 0;
    }
    //check +6dB RDAC1 button
    if (*inc6dBRdac2)
    {
        AD5235_SendCommand((unsigned char) INCREMENT_6dB_CMD, 1, NULL, NULL);
        *inc6dBRdac2 = 0;
    }
    //check -6dB RDAC2 button
    if (*dec6dBRdac2)
    {
        AD5235_SendCommand((unsigned char) DECREMENT_6dB_CMD, 1, NULL, NULL);
        *dec6dBRdac2 = 0;
    }
    //check +1 Step RDAC1 button
    if (*incRdac1)
    {
        AD5235_SendCommand((unsigned char) INCREMENT_1STEP_CMD, 0, NULL, NULL);
        *incRdac1 = 0;
    }
    //check -1 Step RDAC1 button
    if (*decRdac1)
    {
        AD5235_SendCommand((unsigned char) DECREMENT_1STEP_CMD, 0, NULL, NULL);
        *decRdac1 = 0;
    }
    //check +1 Step RDAC2 button
    if (*incRdac2)
    {
        AD5235_SendCommand((unsigned char) INCREMENT_1STEP_CMD, 1, NULL, NULL);
        *incRdac2 = 0;
    }
    //check -1 Step RDAC2 button
    if (*decRdac2)
    {
        AD5235_SendCommand((unsigned char) DECREMENT_1STEP_CMD, 1, NULL, NULL);
        *decRdac2 = 0;
    }
    //check +1 Step RDACs button
    if (*incRdacs)
    {
        AD5235_SendCommand((unsigned char) INCREMENT_1STEP_ALL_CMD, 0, NULL, NULL);
        *incRdacs = 0;
    }
    //check -1 Step RDACs button
    if (*decRdacs) {
        AD5235_SendCommand((unsigned char) DECREMENT_1STEP_ALL_CMD, 0, NULL,
                NULL);
        *decRdacs = 0;
    }
    //check +6dB RDACs button
    if (*inc6dBRdacs)
    {
        AD5235_SendCommand((unsigned char) INCREMENT_6dB_ALL_CMD, 0, NULL, NULL);
        *inc6dBRdacs = 0;
    }
    //check -6dB RDACs button
    if (*dec6dBRdacs)
    {
        AD5235_SendCommand((unsigned char) DECREMENT_6dB_ALL_CMD, 0, NULL, NULL);
        *dec6dBRdacs = 0;
    }
    //compute tolerance
    if (*toleranceFlag)
    {
        short i;
        short temp; //temp is used as intermediary var. for extracting the integer part/fractional part for tolerance
        short val;  //val is used as intermediary variable to compute the fractional part
        unsigned short toleranceRead; //used to store the tolerance value as read from the AD5235
        AD5235_SendCommand((unsigned char) READ_EEM_CMD, 15, NULL, &toleranceRead);
        temp = (toleranceRead << 1); //computing the integer value
        *tolerance = temp >> 9;
        temp = toleranceRead & 0x00FF; //computing the fractional part
        for (i = 0; i < 8; i++)
        {
            val = (temp & (1 << i)) ? 1 : 0;
            *tolerance += val ? (1.0f / ((1 << (8 - i)) * val)) : 0;
        }
        temp = toleranceRead & 0x8FFF; //computing the sign
        *tolerance = temp ? *tolerance : *tolerance * (-1);
        *toleranceFlag = 0;
    }
    //check Send Command button
    if (*cmdFlag)
    {
        AD5235_SendCommand((unsigned char) *command, *cmdAddr, cmdData,
                cmdResponseS);
        *cmdAddrResponse = rxBuffer[0] & 0x0F;
        *cmdResponseC = rxBuffer[0] >> 4;
        *cmdFlag = 0;
    }
    //check Write Protect switch. This controls directly a hardware pin
    if (!*eemWp)
    {
        GPIO_SET_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_WP);
    }
    else
    {
        GPIO_CLEAR_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_WP);
    }
    //\PR button. This controls directly an Hardware Pin
    if (*pr)
    {
        GPIO_CLEAR_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_RESET);
        *pr = 0;
    }
    else
    {
        GPIO_SET_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_RESET);
    }

    return(TRUE);
}
