/**************************************************************************//**
*   @file   main.c
*   @brief  Implementation of main function for FMC-SDP evaluation projects.
*   @author ACozma (andrei.cozma@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 468
******************************************************************************/
/*****************************************************************************/
/***************************** Include Files *********************************/
/*****************************************************************************/
#include <stdio.h>
#include "platform.h"
#include "xil_io.h"
#include "xparameters.h"
#include "../drivers/i2c.h"
#include "../drivers/spi.h"
#include "../drivers/gpio.h"
#include "../uCProbeLib/ucprobe.h"
#if (PROBE_EN == 1)
	#include "../uCProbeLib/probe_com.h"
	#include "../uCProbeLib/probe_rs232.h"
#endif
#include "AD5235.h"
/*****************************************************************************/
/************************** Macros Definitions *******************************/
/*****************************************************************************/
#define AD_Init AD5235_Init
#define AD_UpdateParameters AD5235_UpdateParameters
/*****************************************************************************/
/************************ Variables Definitions ******************************/
/*****************************************************************************/
static volatile unsigned char AD_Select = 0;
/**************************************************************************//**
* @brief Main function.
*
* @return Always returns 0.
******************************************************************************/
int main()
{
	unsigned char prevAD_Select = 0;
	init_platform();
#if (PROBE_EN == 1)
	ProbeCom_Init();
	ProbeRS232_Init(115200);
	ProbeRS232_RxIntEn();
#endif
	while(1)
	{
		if(AD_Select)
		{
			if(prevAD_Select == 0)
			{
				AD_Select = AD_Init();
			}
			prevAD_Select = AD_Select;
		}
		else
		{
			prevAD_Select = 0;
		}
		if(prevAD_Select)
		{
			if(AD_UpdateParameters() == 0)
			{
				AD_Select = 0;
				prevAD_Select = 0;
			}
		}
	}
    cleanup_platform();
    return 0;
}
